# :nodoc:
#
# Original Author:: Nathaniel Talbott.
# Co Original Author:: Kazuhiro NISHIYAMA.
# Author:: arton
# Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
# Copyright:: Copyright (c) 2003 Kazuhiro NISHIYAMA. All rights reserved.
# Copyright:: Copyright (c) 2004 arton. All rights reserved.
# License:: Ruby license.

require 'vr/vruby'
require 'vr/vrcontrol'
require 'vr/vrcomctl'
require 'test/unit/ui/testrunnermediator'
require 'test/unit/ui/testrunnerutilities'
require 'test/unit/collector/dir'
module WMsg
  PBM_SETBARCOLOR      = WM_USER+9
end

module Test
  module Unit
    module UI
      module VRuby # :nodoc:

	GREEN=RGB(0,0x80,0)
	RED=RGB(0xff,0,0)
	OFN_FILEMUSTEXIST = 0x00001000

	class RunnerForm < VRForm
	  include VRMenuUseable

	  def construct
	    self.caption = 'VRuby TestRunner'
	    y = 0
	    addControl(VRStatic, 'c0', 'Suite:', 0, y, 80, 20)
	    addControl(VRStatic, 'suite_name_entry', '', 80, y, 450, 20)
	    addControl(VRButton, 'run_button', 'Run', 530, y, 60, 20)
	    y += 20
	    addControl(VRProgressbar, 'test_progress_bar', '', 0, y, 590, 20)
	    y += 20
	    addControl(VRStatic, 'c1', 'Tests:', 10, y, 60, 20)
	    addControl(VRStatic, 'test_count_label', '0', 70, y, 60, 20)
	    addControl(VRStatic, 'c2', 'Assertions:', 130, y, 80, 20)
	    addControl(VRStatic, 'assertion_count_label', '0', 220, y, 60, 20)
	    addControl(VRStatic, 'c3', 'Failures:', 280, y, 80, 20)
	    addControl(VRStatic, 'failure_count_label', '0', 360, y, 60, 20)
	    addControl(VRStatic, 'c4', 'Errors:', 470, y, 60, 20)
	    addControl(VRStatic, 'error_count_label', '0', 530, y, 60, 20)
	    y += 20
	    addControl(VRListbox, 'fault_list', '', 0, y, 590, 200)
	    y += 200
	    addControl(VRText, 'detail_text', '', 0, y, 590, 120,
	       WStyle::ES_MULTILINE|WStyle::WS_VSCROLL|WStyle::WS_HSCROLL)
	    y += 120
	    addControl(VRStatusbar, 'status_entry', '', 0, y, 600, 20)
	    @status_entry.minheight = 20
	    @red = false
	    menu = newMenu
	    menu.set([ ['&File', [['&Open', 'open'], VRMenu::SEPARATOR,
			   ['E&xit', 'exit']],]])
	    setMenu(menu)
	  end

	  def open_clicked
	    file = openFilenameDialog(['ruby script(*.rb)', '*.rb', 
				       'all(*.*)', '*.*'], 
				      OFN_FILEMUSTEXIST, 'TestCase', 'rb')
	    @runner.opened(file) if !file.nil?
	  end

	  def exit_clicked
	    exit
	  end
	  
	  def suite_name_entry=(caption)
	    @suite_name_entry.caption = caption
	  end

	  def button=(caption)
	    @run_button.caption = caption
	  end

	  def test_count_label=(count)
	    @test_count_label.caption = count
	    @test_progress_bar.position = count
	  end

	  def assertion_count_label=(count)
	    @assertion_count_label.caption = count
	  end

	  def failure_count_label=(count)
	    @failure_count_label.caption = count
	  end

	  def error_count_label=(count)
	    @error_count_label.caption = count
	  end

	  def detail_text=(text)
	    @detail_text.text = text
	  end

	  def status_entry=(text)
	    @status_entry.setTextOf(0, text)
	  end

	  def count=(cnt)
	    @test_progress_bar.setRange(0, cnt)
	  end

	  def red=(bool)
	    @red = bool
	    @test_progress_bar.sendMessage(WMsg::PBM_SETBARCOLOR, 0, bool ? RED : GREEN)
	  end

	  def red?
	    @red
	  end

	  def add_fault(msg)
            @fault_list.addString(msg)
	  end

	  def reset(count)
	    @test_progress_bar.sendMessage(WMsg::PBM_SETBARCOLOR, 0, GREEN)
	    @test_progress_bar.setRange(0, count)
            @red = false

            test_count_label = 0
            assertion_count_label = 0
            failure_count_label = 0
            error_count_label = 0

            @fault_list.clearStrings
	  end

	  def set_runner(runner)
	    @runner = runner
	  end

	  def fault_list_selchanged
	    @runner.show_fault(@fault_list.selectedString)
	  end

	  def run_button_clicked
	    @runner.run_test
	  end

	end

        # Runs a Test::Unit::TestSuite in a VisualuRuby UI. Obviously,
        # this one requires you to have VisualuRuby
        # and the Ruby Swin extension installed.
        class TestRunner
          extend TestRunnerUtilities

          # Creates a new TestRunner and runs the suite.
          def self.run(suite, output_level=NORMAL)
            new(suite).start
          end

	  def suite=(suite)
	    if suite.nil?
	      @suite = nil
            elsif (suite.respond_to?(:suite))
              @suite = suite.suite
            else
              @suite = suite
            end
	  end

          # Creates a new TestRunner for running the passed
          # suite.
          def initialize(initsuite, output_level = NORMAL)
	    self.suite = initsuite
            @result = nil

            @fault_detail_list = []
            @runner = Thread.current
            @restart_signal = Class.new(Exception)
            @view = VRLocalScreen.showForm(RunnerForm, 0, 0, 600, 440)
	    @view.set_runner(self)
	    @viewer = Thread.start do
              @runner.join rescue @runner.run
	      VRLocalScreen.messageloop
	    end
            @viewer.join rescue nil # wait deadlock to handshake
          end

          # Begins the test run.
          def start
            setup_mediator
            attach_to_mediator
            start_ui
            @result
          end

          def run_test
            @runner.raise(@restart_signal)
          end

          def show_fault(fault) # :nodoc:
	    f = @fault_detail_list[fault]
	    if f
	      raw_show_fault(f.long_display)
	    end
          end

	  def opened(file)
	    @suite = Test::Unit::Collector::Dir.new().collect(file)
	    setup_mediator
	    attach_to_mediator
	    @runner.raise(@restart_signal)
	  end

          def start_ui # :nodoc:
            @viewer.run
            running = false
            begin
              loop do
		if @suite.nil?
		  @viewer.join
		else
		  if (running ^= true)
		    @view.button = 'Stop'
		    @mediator.run_suite
		  else
		    @view.button = 'Run'
		    @viewer.join
		    break
		  end
                end
              end
            rescue @restart_signal
              retry
            rescue
	      p $! if $VERBOSE
            end
          end

          private
          def setup_mediator # :nodoc:
            @mediator = TestRunnerMediator.new(@suite)
            suite_name = @suite.to_s
            if ( @suite.kind_of?(Module) )
              suite_name = @suite.name
            end
            @view.suite_name_entry = suite_name
          end

          def attach_to_mediator # :nodoc:
            @mediator.add_listener(TestRunnerMediator::RESET, &method(:reset_ui))
            @mediator.add_listener(TestResult::FAULT, &method(:add_fault))
            @mediator.add_listener(TestResult::CHANGED, &method(:result_changed))
            @mediator.add_listener(TestRunnerMediator::STARTED, &method(:started))
            @mediator.add_listener(TestCase::STARTED, &method(:test_started))
            @mediator.add_listener(TestRunnerMediator::FINISHED, &method(:finished))
          end

          def reset_ui(count) # :nodoc:
            @test_total_count = count.to_f
	    @view.reset(count)
            @fault_detail_list = []
            clear_fault
          end

          def add_fault(fault) # :nodoc:
            if ( ! @view.red? )
              @view.red = true
            end
            @fault_detail_list.push fault
	    @view.add_fault(fault.short_display)
          end

          def raw_show_fault(string) # :nodoc:
            @view.detail_text = string
          end

          def clear_fault # :nodoc:
            raw_show_fault("")
          end

          def result_changed(result) # :nodoc:
            @view.test_count_label = result.run_count
            @view.assertion_count_label = result.assertion_count
            @view.failure_count_label = result.failure_count
            @view.error_count_label = result.error_count
          end

          def started(result) # :nodoc:
            @result = result
            output_status("Started...")
          end

          def test_started(test_name)
            output_status("Running #{test_name}...")
          end

          def finished(elapsed_time)
            output_status("Finished in #{elapsed_time} seconds")
          end

          def output_status(string) # :nodoc:
            @view.status_entry = string
          end

        end
      end
    end
  end
end

if __FILE__ == $0
  if ARGV.empty?
    win = Test::Unit::UI::VRuby::TestRunner.new(nil)
    win.start_ui
  else
    Test::Unit::UI::VRuby::TestRunner.start_command_line_test
  end
end

